#!/bin/bash -e

#
### BEGIN DJIGZO SCRIPT INFO
# Name: backup
# Description: Script that backups/restores the CipherMail database and files specified in backup_files
### END DJIGZO SCRIPT INFO
#
# Author: Martijn Brinkers
#
# Copyright 2008-2022, CipherMail.
#

# set a sane/secure path
PATH='/bin:/usr/bin:/sbin:/usr/sbin'
# it's almost certainly already marked for export, but make sure
export PATH

# remove all aliases (start with \ to prevent unalias from being aliased)
\unalias -a

# clean command hash
hash -r

# set a sane/secure IFS (note this is bash & ksh93 syntax only--not portable!)
IFS=$' \t\n'

# the user under which CipherMail runs
cm_user="djigzo"

version_filename="djigzo-version"

# the default database type for backup/restore
database_type="postgres"

# the current backup version 
this_version="3"

tmp_dir=
action=
backup_file=
backup_password=
files_to_backup=
backup_version=
sql_dump_filename=

GENERAL_EXIT_CODE=100
INCORRECT_USAGE_EXIT_CODE=101
DECRYPTION_FAILED_EXIT_CODE=102
ENCRYPTION_FAILED_EXIT_CODE=103
TEMP_DIR_NOT_SET_EXIT_CODE=104
BACKUP_FILE_NOT_EXIST_EXIT_CODE=105
INCORRECT_BACKUP_FILE_EXIST_EXIT_CODE=106
MKTEMP_FAILED_EXIT_CODE=107
BACKUP_FILE_VERSION_INCORRECT_EXIT_CODE=108
CREATE_TABLES_SCRIPT_NOT_EXIST_EXIT_CODE=109
DROP_TABLES_SCRIPT_NOT_EXIST_EXIT_CODE=110
RESTART_SCRIPT_NOT_EXIST_EXIT_CODE=111
BACKUP_FRAGMENTS_DIR_MODE_FLAGS_INCORRECT_EXIT_CODE=112
INVALID_DATABASE_TYPE_EXIT_CODE=113
MYSQL_PASSWORD_FILE_NOT_EXIST_EXIT_CODE=114
MYSQL_PASSWORD_FILE_NOT_READABLE_EXIT_CODE=115

# include defaults if available
if [[ -f /etc/default/cm-backup ]] ; then
    . /etc/default/cm-backup
fi

if [[ -z $ciphermail_home ]]; then
    ciphermail_home=/usr/share/djigzo
fi

if [[ -z $database_name ]]; then
    database_name="djigzo"
fi

if [[ -z $database_user ]]; then
    database_user="djigzo"
fi


# dir that contains all the fragments with the files to backup
backup_fragments_dir="$ciphermail_home/conf/backup.d"

# check the file mode bits of the fragments dir to make sure it is owned by root, readable and only writable by root
if stat --format="%U %G %a" $backup_fragments_dir != "root root 755" &> /dev/null;
then
    echo "$backup_fragments_dir is not owned by root and/or the mode is not 755" >&2

    exit $BACKUP_FRAGMENTS_DIR_MODE_FLAGS_INCORRECT_EXIT_CODE
fi

# find all backup fragments owned by root
backup_fragments=$(find -L "$backup_fragments_dir" -user root -type f -perm 644 | sort)

# make sure that only newline separates the entries (to support filenames with spaces)
IFS=$'\n'

# create a newline separated list of files to backup
for file_to_backup in $backup_fragments
do
    files_to_backup="$files_to_backup"$(cat "$file_to_backup")$'\n'
done

# set IFS back to a sane value
IFS=$' \t\n'

usage()
{
    echo "Usage: [option...]" >&2
    echo "" >&2
    echo "  -b                      Create backup to std-out" >&2
    echo "  -r <file>               Restore backup" >&2
    echo "  -l <file>               Print a list of files from the backup" >&2
    echo "  -t <database type>      Database type (postgres | mysql)" >&2
    echo "  -p <password>           Backup/restore password" >&2
    echo "  -P                      Read password from std-in" >&2
    echo "  -d                      Include database in backup/restore" >&2
    echo "  -o                      Include other files in backup/restore" >&2
    exit $INCORRECT_USAGE_EXIT_CODE
}

cleanup()
{
    #echo "cleaning up " $tmp_dir >&2

    if [ ! "$tmp_dir" ]; then
        echo "tmp_dir is not set" >&2

        exit $TEMP_DIR_NOT_SET_EXIT_CODE
    fi

    rm -rf $tmp_dir
}

#
# checks whether the backup file exists
# 
check_backup_exists()
{
    if [ ! -f "$backup_file" ]; then
        echo "Backup file does not exist" >&2

        exit $BACKUP_FILE_NOT_EXIST_EXIT_CODE
    fi
}

decryption_failed()
{
    echo "decryption failed" >&2

    exit $DECRYPTION_FAILED_EXIT_CODE
}

encryption_failed()
{
    echo "encryption failed" >&2

    exit $ENCRYPTION_FAILED_EXIT_CODE
}

invalid_backup()
{
    echo "File is not a valid backup file" >&2

    exit $INCORRECT_BACKUP_FILE_EXIST_EXIT_CODE
}

incorrect_version()
{
    echo "Backup file is from a newer version of CipherMail. Supported versions <= $this_version, backup version = $backup_version" >&2

    exit $BACKUP_FILE_VERSION_INCORRECT_EXIT_CODE
}

init()
{
    if [[ "$database_type" == "postgres" ]]; then
        pg_cm_init
    elif [[ "$database_type" == "mysql" ]]; then
        mysql_cm_init
    fi
}

backup()
{
    local tar_file=$tmp_dir/backup.tar

    # create an empty tar to which we will add the files
    tar -cf "$tar_file" --files-from /dev/null

    # create a version file so we can detect that this is a CipherMail backup
    echo "$this_version" > $tmp_dir/$version_filename
    tar -C $tmp_dir -rPf "$tar_file" "$version_filename"

    # check if database backup is excluded
    if [[ $cm_backup_include_database = true ]]; then
        if [[ "$database_type" = "postgres" ]]; then
            # dump the database to a temp file and add it to the tar
            pg_cm_dump "$tmp_dir/$sql_dump_filename"
            tar -C $tmp_dir -rPf "$tar_file" "$sql_dump_filename"
        elif [[ "$database_type" == "mysql" ]]; then
            # dump the database to a temp file and add it to the tar
            mysql_cm_dump "$tmp_dir/$sql_dump_filename"
            tar -C $tmp_dir -rPf "$tar_file" "$sql_dump_filename"
        fi
    fi

    if [[ $cm_backup_include_other = true ]]; then
        # add the files we want to backup
        for file_to_backup in $files_to_backup; do

            local filename=$(echo -n "$file_to_backup" | cut -f 1 -d#)

            # skip lines that start with #        
            if [ -z "$filename" ]; then
                continue
            fi

            # check if file exists (with possible wildcard) 
            if ls $filename > /dev/null 2>&1; then
                    tar -rPf "$tar_file" $filename || true
            fi
        done
    fi

    local local_backup_file=$tar_file

    # if a password is specified we will GPG encrypt the tar
    if [ "$backup_password" ]; then
        echo "$backup_password" | gpg -q --yes -c --no-tty --batch --passphrase-fd 0 -o "$tar_file".gpg "$tar_file" 2> /dev/null || encryption_failed
        local_backup_file="$tar_file".gpg
    else
        gzip "$tar_file"
        local_backup_file="$tar_file".gz
    fi

    # for security reasons we will not write the backup to a file but will write the backup to std out
    # otherwise someone might overwrite random files because the backup script is run as root
    cat "$local_backup_file"
}

#
# decrypt the backup_file if password is set. After decryption backup_file will be set
# to the decrypted TAR file
# 
decrypt()
{
    # if a password is specified we assume the file is a GPG encrypted tar
    if [ "$backup_password" ]; then
        # decrypt to backup.tar in de temp dir
        echo "$backup_password" | gpg -q -d --no-tty --batch --passphrase-fd 0 -o $tmp_dir/backup.tar "$backup_file" 2> /dev/null || decryption_failed

        backup_file=$tmp_dir/backup.tar
    fi
}

# gunzip's the backup if it is gzip'd
uncompress()
{
    # check if the file is gzip'd
    if gunzip -t "$backup_file" 2> /dev/null; then
        # gunzip the file to a tar file in the temp dir
        gunzip --stdout "$backup_file" > $tmp_dir/backup.tar
        backup_file=$tmp_dir/backup.tar
    fi
}

#
# checks whether the backup tar contains the version string
# 
check_valid_backup()
{
    tar -tPf "$backup_file" | grep "^$version_filename\s*\$" >&2 > /dev/null || invalid_backup
    
    # read the backup version (backup_version should be a global variable)
    backup_version=$(tar --to-stdout -xf "$backup_file" "$version_filename" | tr -dc "1234567890" 2> /dev/null);    

    local allow_restore=$(echo $backup_version|awk '{ if ($1 <= '$this_version') {print "1"}}')
    
    if [ ! $allow_restore ]; then
        incorrect_version
    fi    
}

list()
{
    [ "$backup_file" ] || usage

    check_backup_exists
    decrypt
    uncompress
    check_valid_backup
    
    tar -tPf "$backup_file" >&1
}

restore()
{
    [ "$backup_file" ] || usage

    check_backup_exists
    decrypt
    uncompress
    check_valid_backup

    # we will stop postfix to make sure no email comes in while restoring
    systemctl stop postfix || true

    if [[ $cm_backup_include_other = true ]]; then
        # restore the files we need
        for file_to_backup in $files_to_backup; do

            local filename=$(echo -n "$file_to_backup" |  cut -f 1 -d#)
            
            # skip lines that start with #        
            if [ -z "$filename" ]; then
                continue
            fi
            
            local own=$(echo -n "$file_to_backup" |  cut -f 2 -d#)
            local mod=$(echo -n "$file_to_backup" |  cut -f 3 -d#)

            if tar --wildcards -xPof "$backup_file" $filename; then
                chown "$own" $filename
                chmod "$mod" $filename
            fi
        done
    fi

    # check if database restore is excluded
    if [[ $cm_backup_include_database = true ]]; then
        # extract the sql file to temp folder and restore the database.
        # it might be that the backup does not contain the SQL script so only continue if it can be extracted
        tar -C "$tmp_dir" -xPf "$backup_file" "$sql_dump_filename" || true

        if [[ -f "$tmp_dir/$sql_dump_filename" ]]; then
            if [[ "$database_type" == "postgres" ]]; then
                pg_cm_restore_backup
            elif [[ "$database_type" == "mysql" ]]; then
                mysql_cm_restore_backup
            fi
        fi
    fi

    # Postfix was stopped when starting the restore so it must be restarted
    systemctl start postfix || true

    # create the restart required file so the GUI shows a restart required warning
    touch /var/run/djigzo-restart-required
}

validate_database_type()
{
    if [[ "$database_type" != "postgres" && "$database_type" != "mysql" && "$database_type" != "oracle" ]]; then
        echo "Invalid database type. Only postgres, mysql and oracle are supported." >&2

        exit $INVALID_DATABASE_TYPE_EXIT_CODE
    fi
}

mysql_cm_init()
{
    sql_dump_filename="ciphermail.mysql.sql"
}

mysql_cm_dump()
{
    local dump_file=$1

    mysqldump "$database_name" > "$dump_file"
}

mysql_cm_restore_backup()
{
    # make sure the restore user can read the dump
    chown "$cm_user":"$cm_user" "$tmp_dir"
    chown "$cm_user":"$cm_user" "$tmp_dir"/"$sql_dump_filename"

    mysql "$database_name" < "$tmp_dir"/"$sql_dump_filename"
}

pg_cm_init()
{
    sql_dump_filename="djigzo.sql"

    # the SQL scripts
    pg_create_tables_script="$ciphermail_home/conf/database/sql/djigzo.sql"
    pg_drop_tables_script="$ciphermail_home/conf/database/sql/djigzo-drop-tables.sql"

    pg_create_tables_dir="$ciphermail_home/conf/database/sql/sql.d"
    pg_drop_tables_dir="$ciphermail_home/conf/database/sql/sql.drop.d"

    if [ ! -f "$pg_create_tables_script" ]; then
        echo "Create tables script $pg_create_tables_script does not exist" >&2

        exit $CREATE_TABLES_SCRIPT_NOT_EXIST_EXIT_CODE
    fi

    if [ ! -f "$pg_drop_tables_script" ]; then
        echo "Drop tables script $pg_drop_tables_script does not exist" >&2

        exit $DROP_TABLES_SCRIPT_NOT_EXIST_EXIT_CODE
    fi
}

pg_cm_dump()
{
    local dump_file=$1

    sudo -u postgres pg_dump "$database_name" > $dump_file
}

pg_cm_restore_backup()
{
    # make sure postgres user can read the dump
    chown postgres:postgres "$tmp_dir"
    chown postgres:postgres "$tmp_dir"/"$sql_dump_filename"

    pg_cm_dropTables
        
    pg_cm_get_restore_extra_params
    
    if [ $backup_version -eq "1" ]; then
       sudo -u postgres pg_restore $pg_restore_extra_params -e -d "$database_name" "$tmp_dir"/"$sql_dump_filename"
    else
       sudo -u postgres psql "$database_name" < "$tmp_dir"/"$sql_dump_filename"
    fi
    
    # we import the SQL to restore tables that were not in the imported sql
    sudo -u "$database_user" psql "$database_name" < "$pg_create_tables_script" || true

    # import all the sql from the .d dir
    find $pg_create_tables_dir -name \*.sql | xargs cat | sudo -u "$database_user" psql "$database_name" || true
}

pg_cm_get_restore_extra_params()
{
    local pg_82=8.2
    local pg_current=$(pg_restore --version | cut -d' ' -f 3)

    pg_restore_extra_params=
    
    # Note: --single-transaction is only supported PostgreSQL >= 8.2    
    pg_restore_extra_params=$(echo $pg_current|awk '{ if ($1 >= '$pg_82') {print "--single-transaction"}}')
}

pg_cm_dropTables()
{
    sudo -u postgres psql "$database_name" < "$pg_drop_tables_script" || true

    # drop all tables defined in the .drop.d dir
    find "$pg_drop_tables_dir" -name \*.sql | xargs cat | sudo -u postgres psql "$database_name" || true
}

[ $# -ne 0 ] || usage

# create a temp dir
tmp_dir=$(mktemp -d) || exit $MKTEMP_FAILED_EXIT_CODE

# create a trap that will cleanup all temporary resources
trap cleanup ABRT EXIT HUP INT QUIT

while getopts 'r:bl:p:Pt:do' OPTION
do
    case $OPTION in
    r)  action="r"
        backup_file="$OPTARG"
        ;;
    b)  action="b"
        ;;
    l)  action="l"
        backup_file="$OPTARG"
        ;;
    p)  backup_password="$OPTARG"
        ;;
    P)  backup_password=$(cat) # read password from std-in
        ;;
    t)  database_type="$OPTARG"
        validate_database_type
        ;;
    d)  cm_backup_include_database=true
        ;;
    o)  cm_backup_include_other=true
        ;;
    ?)  usage
        ;;
    esac
done

shift $(($OPTIND - 1))

init

case $action in
    r)  restore
        ;;
    b)  backup
        ;;
    l)  list
        ;;
esac

exit 0
