#!/bin/bash
#
# Script that checks whether the CipherMail backend is running. Returns "ok" if running and "fail" if not running.
# 
# For example, the script be queried over the network with help of xinetd. xinetd config: 
#  
#service ciphermail
#{
#        port            = 777
#        disable         = no
#        type            = UNLISTED
#        socket_type     = stream
#        protocol        = tcp
#        user            = djigzo
#        wait            = no
#        instances       = 4
#        server          = /usr/share/djigzo/scripts/check_backend.sh
#        server_args     = --running
#        only_from       = 192.168.178.20
#}                                                                               
# 
# Copyright 2008-2020. CipherMail.
#

# set a sane/secure path
PATH='/bin:/usr/bin:/sbin:/usr/sbin'
# it's almost certainly already marked for export, but make sure
export PATH

# remove all aliases (start with \ to prevent unalias from being aliased)
\unalias -a

# clean command hash
hash -r

# set a sane/secure IFS (note this is bash & ksh93 syntax only--not portable!)
IFS=$' \t\n'

# Include djigzo defaults if available
if [ -f /etc/default/djigzo ] ; then
    . /etc/default/djigzo
fi

if [ ! "$DJIGZO_HOME" ]; then
    DJIGZO_HOME=/usr/share/djigzo
fi

usage()
{
    echo "usage: check_backend.sh (--running | --mpa-queue-size <repository> | --mta-queue-size | --crl-store-size | --cert-store-size | --cert-request-queue-size | --sms-queue-size | --user-store-size | --pgp-keyring-size | --quarantine-queue-size)" >&2

    exit 1
}

classpath="$DJIGZO_HOME/lib/*"
java_main_class="mitm.application.djigzo.tools.Monitor"

checkRunning()
{
    running=$(java -cp "$classpath" $java_main_class --running 2> /dev/null)
    
    exit_code=$?
    
    if [ $exit_code -gt 0 ] || [ "$running" != "true" ]; then
      echo "fail"
    else
      echo "ok"
    fi
}

printMPAQueueSize()
{
    result=$(java -cp "$classpath" $java_main_class --mpa-queue-size "$@" 2>&1)
    
    exit_code=$?
    
    if [ $exit_code -gt 0 ]; then
      echo "Error: $result"
      exit $exit_code;
    else
      echo "$result"
    fi
}

printMTAQueueSize()
{
    result=$(java -cp "$classpath" $java_main_class --mta-queue-size 2>&1)
    
    exit_code=$?
    
    if [ $exit_code -gt 0 ]; then
      echo "Error: $result"
      exit $exit_code;
    else
      echo "$result"
    fi
}

printCRLStoreSize()
{
    result=$(java -cp "$classpath" $java_main_class --crl-store-size 2>&1)
    
    exit_code=$?
    
    if [ $exit_code -gt 0 ]; then
      echo "Error: $result"
      exit $exit_code;
    else
      echo "$result"
    fi
}

printCertStoreSize()
{
    result=$(java -cp "$classpath" $java_main_class --cert-store-size 2>&1)
    
    exit_code=$?
    
    if [ $exit_code -gt 0 ]; then
      echo "Error: $result"
      exit $exit_code;
    else
      echo "$result"
    fi
}

printCertRequestQueueSize()
{
    result=$(java -cp "$classpath" $java_main_class --cert-request-queue-size 2>&1)
    
    exit_code=$?
    
    if [ $exit_code -gt 0 ]; then
      echo "Error: $result"
      exit $exit_code;
    else
      echo "$result"
    fi
}

printSMSQueueSize()
{
    result=$(java -cp "$classpath" $java_main_class --sms-queue-size 2>&1)
    
    exit_code=$?
    
    if [ $exit_code -gt 0 ]; then
      echo "Error: $result"
      exit $exit_code;
    else
      echo "$result"
    fi
}

printUserStoreSize()
{
    result=$(java -cp "$classpath" $java_main_class --user-store-size 2>&1)
    
    exit_code=$?
    
    if [ $exit_code -gt 0 ]; then
      echo "Error: $result"
      exit $exit_code;
    else
      echo "$result"
    fi
}

printPGPKeyringSize()
{
    result=$(java -cp "$classpath" $java_main_class --pgp-keyring-size 2>&1)
    
    exit_code=$?
    
    if [ $exit_code -gt 0 ]; then
      echo "Error: $result"
      exit $exit_code;
    else
      echo "$result"
    fi
}

printQuarantineQueueSize()
{
    result=$(java -cp "$classpath" $java_main_class --quarantine-queue-size 2>&1)
    
    exit_code=$?
    
    if [ $exit_code -gt 0 ]; then
      echo "Error: $result"
      exit $exit_code;
    else
      echo "$result"
    fi
}

if [ $# -lt 1 ]; then
    usage
fi

COMMAND="$1"

shift

case "$COMMAND" in
    --running)
        checkRunning $@ 
    ;;
    --mpa-queue-size)
        printMPAQueueSize $@ 
    ;;
    --mta-queue-size)
        printMTAQueueSize $@ 
    ;;
    --crl-store-size)
        printCRLStoreSize $@ 
    ;;
    --cert-store-size)
        printCertStoreSize $@ 
    ;;
    --cert-request-queue-size)
        printCertRequestQueueSize $@ 
    ;;
    --sms-queue-size)
        printSMSQueueSize $@ 
    ;;
    --user-store-size)
        printUserStoreSize $@ 
    ;;
    --pgp-keyring-size)
        printPGPKeyringSize $@ 
    ;;
    --quarantine-queue-size)
        printQuarantineQueueSize $@ 
    ;;
    *)
        usage
esac

exit 0
