#!/bin/bash -e

# Script that tries to find a script from the scrips dir (scripts.d) with the name given as the first parameter.
# The scripts should be owned by root, not writable by group or other and executable. The name of the script is
# extracted from a specialized part of the script that looks as follows:
#
# ### BEGIN DJIGZO SCRIPT INFO
# Name: test
# Description: some description
# ### END DJIGZO SCRIPT INFO
# 
# The first parameter of this script is the name that will be matched against the Name parameter. The script files
# are sorted with sort and the first match (i.e., the first script where the Name parameter matches the parameter)
# is executed with the rest of the parameters as input. The exit code of this script will be the exit code of the
# script that is executed. If not matching script is found, exit code 255 will be returned
#
# Author: Martijn Brinkers
#
# Copyright 2012-2021, CipherMail.

SCRIPT_NOT_FOUND_EXIT_CODE=255
SCRIPTS_DIR_MODE_FLAGS_INCORRECT_EXIT_CODE=254
SCRIPT_NAME_MISSING_EXIT_CODE=253

scripts_d_dir=$(dirname "$0")"/scripts.d"

name_to_execute=$1

if [ -z "$name_to_execute" ];
then
    echo "name of script to execute is missing" >&2

    exit $SCRIPT_NAME_MISSING_EXIT_CODE
fi

shift

# check the file mode bits of the scripts dir to make sure it is owned by root, readable and only writable by root
if stat --format="%U %G %a" $scripts_d_dir != "root root 755" &> /dev/null;
then
    echo "$scripts_d_dir is not owned by root and/or the mode is not 755" >&2

    exit $SCRIPTS_DIR_MODE_FLAGS_INCORRECT_EXIT_CODE
fi

# find all files from scripts.d
script_files=$(find -L "$scripts_d_dir" -type f | sort)

for script_file in $script_files
do
    script_name=$(grep -E -A1 -B1 "^### (BEGIN|END) DJIGZO SCRIPT INFO" "$script_file" | sed -n '/# Name:.*/p' | awk -F: '{gsub(/^[ \t]+|[ \t]+$/,"",$2); print $2 }')

    if [ "$script_name" == "$name_to_execute" ]; 
    then
        $script_file "$@"   

        exit $?;
    fi
done

echo "script with name $name_to_execute not found" >&2

exit $SCRIPT_NOT_FOUND_EXIT_CODE

