#!/bin/bash -e

# script that "injects" a log into the MPA log
# Copyright CipherMail 2019 

# set a sane/secure path
PATH='/bin:/usr/bin:/sbin:/usr/sbin:/usr/local/bin'
# it's almost certainly already marked for export, but make sure
export PATH

# remove all aliases (start with \ to prevent unalias from being aliased)
\unalias -a

# clean command hash
hash -r

# set a sane/secure IFS (note this is bash & ksh93 syntax only--not portable!)
IFS=$' \t\n'

GENERAL_EXIT_CODE=100

script_name=$(basename "$0")

webservice_username="admin"
webservice_password="password"
webservice_host="127.0.0.1"
webservice_port="9009"
priority="info"

usage()
{
    echo "Usage: $script_name --log <value> --category <value> [--info] [--warn] [--error]" >&2
    echo "" >&2
    echo "    -h                        show usage" >&2
    echo "    --help                    show usage" >&2
    echo "    --log <value>             the content to log" >&2
    echo "    --category <value>        the log category" >&2
    echo "    --info                    log with info priority (the default)" >&2
    echo "    --warn                    log with warn priority" >&2
    echo "    --error                   log with error priority" >&2
    
    exit 1
}

exit_with_error()
{
    echo "$1" >&2

    local exit_code="$2"

    if [ -z "$exit_code" ]; then
        exit_code=$GENERAL_EXIT_CODE;
    fi

    exit "$exit_code"
}

html_encode()
{
    echo "$1" | sed 's/&/\&amp;/g; s/</\&lt;/g; s/>/\&gt;/g; s/"/\&quot;/g; s/'"'"'/\&#39;/g'
}

[ $# -ne 0 ] || usage

# Note that we use `"$@"' to let each command-line parameter expand to a 
# separate word. The quotes around `$@' are essential!
# We need GETOPT_TEMP as the `eval set --' would nuke the return value of getopt.
GETOPT_TEMP=$(getopt -o h --long "help, log:, category:, info, warn, error" \
    -n "$script_name" -- "$@")

if [ $? != 0 ] ; then echo "Terminating..." >&2 ; exit 1 ; fi

# Note the quotes around $GETOPT_TEMP: they are essential!
eval set -- "$GETOPT_TEMP"

while true ; do
    case "$1" in
        -h|--help)      usage ;;
        --log)          log="$2" ; shift 2 ;;
        --category)     category="$2" ; shift 2 ;;
        --info)         priority="info" ; shift 1 ;;
        --warn)         priority="warn" ; shift 1 ;;
        --error)        priority="error" ; shift 1 ;;
        --)             shift ; break ;;
        *) echo "Internal error!" ; exit 1 ;;
    esac
done

if [[ -z "$log" ]]; then
    exit_with_error "log parameter is missing"
fi

if [[ -z "$category" ]]; then
    exit_with_error "category parameter is missing"
fi

# we need to HTML escape the input

log=$(html_encode "$log")
category=$(html_encode "$category")

soap_request="<soap:Envelope xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\"><soap:Body><ns2:$priority xmlns:ns2=\"http://ws.djigzo.application.mitm/\"><category>$category</category><message>$log</message></ns2:$priority></soap:Body></soap:Envelope>"

curl -X POST -H "Content-Type: application/xop+xml" -H 'SOAPAction: ""' -u "$webservice_username":"$webservice_password" --data-binary "$soap_request" http://$webservice_host:$webservice_port/eventloggerws > /dev/null 2>&1

exit 0



