#!/bin/bash

set -eE

#
### BEGIN DJIGZO SCRIPT INFO
# Name: system
# Description: Script for restarting the gateway, reboot the server or halt
### END DJIGZO SCRIPT INFO
#
# Author: Martijn Brinkers
#
# Copyright 2008-2021, CipherMail.
#

# set a sane/secure path
PATH='/bin:/usr/bin:/sbin:/usr/sbin'
# it's almost certainly already marked for export, but make sure
export PATH

# remove all aliases (start with \ to prevent unalias from being aliased)
\unalias -a

# clean command hash
hash -r

# set a sane/secure IFS (note this is bash & ksh93 syntax only--not portable!)
IFS=$' \t\n'

# delay for background process to allow the GUI to finish
background_delay="4"

# include defaults if available
if [ -f /etc/default/cm-system ] ; then
    . /etc/default/cm-system
fi

cleanup()
{
    if [[ -f "$tmpfile" ]]; then
        rm $tmpfile
    fi
}

exit_with_error()
{
    echo "$1" >&2

    local exit_code="$2"

    if [ -z "$exit_code" ]; then
        exit_code=1;
    fi

    exit "$exit_code"
}

restart_all_services()
{
    systemd-run bash -c " \
    sleep $background_delay; \
    systemctl stop ciphermail-gateway-backend; \
    systemctl restart postfix; \
    systemctl restart tomcat*; \
    systemctl start ciphermail-gateway-backend;" &
}

restart_web_server()
{
    systemd-run bash -c " \
    sleep $background_delay; \
    systemctl restart tomcat*;" &
}

reboot_server()
{
    systemd-run bash -c " \
    sleep $background_delay; \
    reboot;" &
}

shutdown_server()
{
    systemd-run bash -c " \
    sleep $background_delay; \
    poweroff;" &
}

import_system_root()
{
    # write std in to tmp file
    tmpfile=$(mktemp) || exit 1

    # check if the cert is valid and output the first one
    cat | openssl x509 > "$tmpfile"

    # create a sha256 hash from the cert file to create a unique filename
    filename="$(cat $tmpfile | sha256sum | cut -d " " -f 1).crt"

    if [[ -d /usr/local/share/ca-certificates ]]; then
        # Debian based system
        cp $tmpfile /usr/local/share/ca-certificates/$filename
        chmod 644 /usr/local/share/ca-certificates/$filename
        update-ca-certificates
    elif [[ -d /etc/pki/ca-trust/source/anchors ]]; then
        # Redhat based system
        cp $tmpfile /etc/pki/ca-trust/source/anchors/$filename
        chmod 644 /etc/pki/ca-trust/source/anchors/$filename
        update-ca-trust
    else
        exit_with_error "Unknown OS"
    fi
}

usage()
{
    echo "usage: system.sh (restart|restart-web-server|reboot|shutdown|import-system-root)" >&2

    exit 1
}

if [ $# -lt 1 ]; then
    usage
fi

# create a trap that will cleanup all temporary resources
trap cleanup ABRT EXIT HUP INT QUIT

command="$1"

shift 

case "$command" in
    restart)
        restart_all_services
    ;;
    restart-web-server)
        restart_web_server
    ;;
    reboot)
        reboot_server
    ;;
    shutdown)
        shutdown_server
    ;;
    import-system-root)
        import_system_root
    ;;
    *)
        usage
esac

exit 0

